use super::*;

pub struct DirectoryIterator<'a> {
    parent: &'a Directory,
    state:  DirectoryIteratorState<'a>
}

impl<'a> DirectoryIterator<'a> {
    pub fn new(parent: &Directory) -> DirectoryIterator {
        DirectoryIterator{parent, state: DirectoryIteratorState::Root}
    }
}

impl<'a> std::iter::Iterator for DirectoryIterator<'a> {
    type Item = DirectoryIteratorElement<'a>;

    fn next(&mut self) -> Option<DirectoryIteratorElement<'a>> {
        match std::mem::replace(&mut self.state, DirectoryIteratorState::Done) {
            DirectoryIteratorState::Root => {
                self.state = DirectoryIteratorState::Data(self.parent.data.iter());
                return Some(DirectoryIteratorElement::Directory(&self.parent));
            }
            DirectoryIteratorState::Data(mut iter) => {
                if let Some(data) = iter.next() {
                    self.state = DirectoryIteratorState::Data(iter);
                    return Some(DirectoryIteratorElement::Data(data));
                }

                else {
                    self.state = DirectoryIteratorState::Directory(self.parent.dirs.iter());
                    return self.next();
                }
            },
            DirectoryIteratorState::Directory(mut iter) => {
                if let Some(dir) = iter.next() {
                    self.state = DirectoryIteratorState::SubDir((iter, std::boxed::Box::new(dir.iter())));
                    return self.next();
                }

                else {
                    return self.next();
                }
            },
            DirectoryIteratorState::SubDir((base_iter, mut iter)) => {
                if let Some(element) = iter.next() {
                    self.state = DirectoryIteratorState::SubDir((base_iter, iter));
                    return Some(element);
                }

                else {
                    self.state = DirectoryIteratorState::Directory(base_iter);
                    return self.next();
                }
            },
            DirectoryIteratorState::Done => {
                return None;
            }
        }
    }
}

enum DirectoryIteratorState<'a> {
    Root,
    Data(std::slice::Iter<'a, Data>),
    Directory(std::slice::Iter<'a, Directory>),
    SubDir((std::slice::Iter<'a, Directory>, std::boxed::Box<DirectoryIterator<'a>>)),
    Done
}

pub enum DirectoryIteratorElement<'a> {
    Directory(&'a Directory),
    Data(&'a Data),
}